@@ -21,9 +21,9 @@ import sys
 
 from osc_lib.command import command
 from osc_lib import utils
-import six
 
 from openstackclient.i18n import _
+from openstackclient.network import common
 
 
 LOG = logging.getLogger(__name__)
@@ -161,6 +161,13 @@ class BaseQuota(object):
                 raise
         return quota._info
 
+    def _network_quota_to_dict(self, network_quota):
+        if type(network_quota) is not dict:
+            dict_quota = network_quota.to_dict()
+        else:
+            dict_quota = network_quota
+        return {k: v for k, v in dict_quota.items() if v is not None}
+
     def get_network_quota(self, parsed_args):
         quota_class = (
             parsed_args.quota_class if 'quota_class' in parsed_args else False)
@@ -174,13 +181,11 @@ class BaseQuota(object):
             client = self.app.client_manager.network
             if default:
                 network_quota = client.get_quota_default(project)
-                if type(network_quota) is not dict:
-                    network_quota = network_quota.to_dict()
+                network_quota = self._network_quota_to_dict(network_quota)
             else:
                 network_quota = client.get_quota(project,
                                                  details=detail)
-                if type(network_quota) is not dict:
-                    network_quota = network_quota.to_dict()
+                network_quota = self._network_quota_to_dict(network_quota)
                 if detail:
                     # NOTE(slaweq): Neutron returns values with key "used" but
                     # Nova for example returns same data with key "in_use"
@@ -274,9 +279,18 @@ class ListQuota(command.Lister, BaseQuota):
         return parser
 
     def take_action(self, parsed_args):
-        projects = self.app.client_manager.identity.projects.list()
         result = []
-        project_ids = [getattr(p, 'id', '') for p in projects]
+        project_ids = []
+        if parsed_args.project is None:
+            for p in self.app.client_manager.identity.projects.list():
+                project_ids.append(getattr(p, 'id', ''))
+        else:
+            identity_client = self.app.client_manager.identity
+            project = utils.find_resource(
+                identity_client.projects,
+                parsed_args.project,
+            )
+            project_ids.append(getattr(project, 'id', ''))
 
         if parsed_args.compute:
             if parsed_args.detail:
@@ -457,18 +471,37 @@ class ListQuota(command.Lister, BaseQuota):
         return ((), ())
 
 
-class SetQuota(command.Command):
+class SetQuota(common.NetDetectionMixin, command.Command):
     _description = _("Set quotas for project or class")
 
     def _build_options_list(self):
-        if self.app.client_manager.is_network_endpoint_enabled():
-            return itertools.chain(COMPUTE_QUOTAS.items(),
-                                   VOLUME_QUOTAS.items(),
-                                   NETWORK_QUOTAS.items())
-        else:
-            return itertools.chain(COMPUTE_QUOTAS.items(),
-                                   VOLUME_QUOTAS.items(),
-                                   NOVA_NETWORK_QUOTAS.items())
+        help_fmt = _('New value for the %s quota')
+        # Compute and volume quota options are always the same
+        rets = [(k, v, help_fmt % v) for k, v in itertools.chain(
+            COMPUTE_QUOTAS.items(),
+            VOLUME_QUOTAS.items(),
+        )]
+        # For docs build, we want to produce helps for both neutron and
+        # nova-network options. They overlap, so we have to figure out which
+        # need to be tagged as specific to one network type or the other.
+        if self.is_docs_build:
+            # NOTE(efried): This takes advantage of the fact that we know the
+            # nova-net options are a subset of the neutron options. If that
+            # ever changes, this algorithm will need to be adjusted accordingly
+            inv_compute = set(NOVA_NETWORK_QUOTAS.values())
+            for k, v in NETWORK_QUOTAS.items():
+                _help = help_fmt % v
+                if v not in inv_compute:
+                    # This one is unique to neutron
+                    _help = self.enhance_help_neutron(_help)
+                rets.append((k, v, _help))
+        elif self.is_neutron:
+            rets.extend(
+                [(k, v, help_fmt % v) for k, v in NETWORK_QUOTAS.items()])
+        elif self.is_nova_network:
+            rets.extend(
+                [(k, v, help_fmt % v) for k, v in NOVA_NETWORK_QUOTAS.items()])
+        return rets
 
     def get_parser(self, prog_name):
         parser = super(SetQuota, self).get_parser(prog_name)
@@ -484,19 +517,24 @@ class SetQuota(command.Command):
             default=False,
             help=_('Set quotas for <class>'),
         )
-        for k, v in self._build_options_list():
+        for k, v, h in self._build_options_list():
             parser.add_argument(
                 '--%s' % v,
                 metavar='<%s>' % v,
                 dest=k,
                 type=int,
-                help=_('New value for the %s quota') % v,
+                help=h,
             )
         parser.add_argument(
             '--volume-type',
             metavar='<volume-type>',
             help=_('Set quotas for a specific <volume-type>'),
         )
+        parser.add_argument(
+            '--force',
+            action='store_true',
+            help=_('Force quota update (only supported by compute)')
+        )
         return parser
 
     def take_action(self, parsed_args):
@@ -510,6 +548,9 @@ class SetQuota(command.Command):
             if value is not None:
                 compute_kwargs[k] = value
 
+        if parsed_args.force:
+            compute_kwargs['force'] = True
+
         volume_kwargs = {}
         for k, v in VOLUME_QUOTAS.items():
             value = getattr(parsed_args, k, None)
@@ -630,7 +671,7 @@ class ShowQuota(command.ShowOne, BaseQuota):
         for k, v in itertools.chain(
                 COMPUTE_QUOTAS.items(), NOVA_NETWORK_QUOTAS.items(),
                 VOLUME_QUOTAS.items(), NETWORK_QUOTAS.items()):
-            if not k == v and info.get(k):
+            if not k == v and info.get(k) is not None:
                 info[v] = info[k]
                 info.pop(k)
 
@@ -643,4 +684,4 @@ class ShowQuota(command.ShowOne, BaseQuota):
             project_name = project_info['name']
             info['project_name'] = project_name
 
-        return zip(*sorted(six.iteritems(info)))
+        return zip(*sorted(info.items()))
