@@ -13,8 +13,6 @@
 
 """IP Floating action implementations"""
 
-from osc_lib.cli import format_columns
-from osc_lib.command import command
 from osc_lib import utils
 from osc_lib.utils import tags as _tag
 
@@ -25,7 +23,6 @@ from openstackclient.network import sdk_utils
 
 
 _formatters = {
-    'location': format_columns.DictColumn,
     'port_details': utils.format_dict,
 }
 
@@ -96,7 +93,8 @@ def _get_attrs(client_manager, parsed_args):
     return attrs
 
 
-class CreateFloatingIP(common.NetworkAndComputeShowOne):
+class CreateFloatingIP(common.NetworkAndComputeShowOne,
+                       common.NeutronCommandWithExtraArgs):
     _description = _("Create floating IP")
 
     def update_parser_common(self, parser):
@@ -177,6 +175,8 @@ class CreateFloatingIP(common.NetworkAndComputeShowOne):
 
     def take_action_network(self, client, parsed_args):
         attrs = _get_attrs(self.app.client_manager, parsed_args)
+        attrs.update(
+            self._parse_extra_properties(parsed_args.extra_properties))
         with common.check_missing_extension_if_error(
                 self.app.client_manager.network, attrs):
             obj = client.create_ip(**attrs)
@@ -392,7 +392,7 @@ class ListFloatingIP(common.NetworkAndComputeLister):
                 ) for s in data))
 
 
-class SetFloatingIP(command.Command):
+class SetFloatingIP(common.NeutronCommandWithExtraArgs):
     _description = _("Set floating IP Properties")
 
     def get_parser(self, prog_name):
@@ -412,6 +412,11 @@ class SetFloatingIP(command.Command):
             help=_("Fixed IP of the port "
                    "(required only if port has multiple IPs)")
         )
+        parser.add_argument(
+            '--description',
+            metavar='<description>',
+            help=_('Set floating IP description')
+        )
         qos_policy_group = parser.add_mutually_exclusive_group()
         qos_policy_group.add_argument(
             '--qos-policy',
@@ -443,6 +448,9 @@ class SetFloatingIP(command.Command):
         if parsed_args.fixed_ip_address:
             attrs['fixed_ip_address'] = parsed_args.fixed_ip_address
 
+        if parsed_args.description:
+            attrs['description'] = parsed_args.description
+
         if parsed_args.qos_policy:
             attrs['qos_policy_id'] = client.find_qos_policy(
                 parsed_args.qos_policy, ignore_missing=False).id
@@ -450,6 +458,9 @@ class SetFloatingIP(command.Command):
         if 'no_qos_policy' in parsed_args and parsed_args.no_qos_policy:
             attrs['qos_policy_id'] = None
 
+        attrs.update(
+            self._parse_extra_properties(parsed_args.extra_properties))
+
         if attrs:
             client.update_ip(obj, **attrs)
 
@@ -484,7 +495,7 @@ class ShowFloatingIP(common.NetworkAndComputeShowOne):
         return (columns, data)
 
 
-class UnsetFloatingIP(command.Command):
+class UnsetFloatingIP(common.NeutronCommandWithExtraArgs):
     _description = _("Unset floating IP Properties")
 
     def get_parser(self, prog_name):
@@ -520,6 +531,8 @@ class UnsetFloatingIP(command.Command):
             attrs['port_id'] = None
         if parsed_args.qos_policy:
             attrs['qos_policy_id'] = None
+        attrs.update(
+            self._parse_extra_properties(parsed_args.extra_properties))
 
         if attrs:
             client.update_ip(obj, **attrs)
