@@ -21,6 +21,7 @@ from osc_lib import utils
 
 from openstackclient.i18n import _
 from openstackclient.identity import common as identity_common
+from openstackclient.network import common
 from openstackclient.network import sdk_utils
 
 
@@ -90,7 +91,7 @@ def _get_attrs(client_manager, parsed_args):
 
 # TODO(abhiraut): Use the SDK resource mapped attribute names once the
 # OSC minimum requirements include SDK 1.0.
-class CreateNetworkRBAC(command.ShowOne):
+class CreateNetworkRBAC(command.ShowOne, common.NeutronCommandWithExtraArgs):
     _description = _("Create network RBAC policy")
 
     def get_parser(self, prog_name):
@@ -150,6 +151,8 @@ class CreateNetworkRBAC(command.ShowOne):
     def take_action(self, parsed_args):
         client = self.app.client_manager.network
         attrs = _get_attrs(self.app.client_manager, parsed_args)
+        attrs.update(
+            self._parse_extra_properties(parsed_args.extra_properties))
         obj = client.create_rbac_policy(**attrs)
         display_columns, columns = _get_columns(obj)
         data = utils.get_item_properties(obj, columns)
@@ -253,7 +256,7 @@ class ListNetworkRBAC(command.Lister):
 
 # TODO(abhiraut): Use the SDK resource mapped attribute names once the
 # OSC minimum requirements include SDK 1.0.
-class SetNetworkRBAC(command.Command):
+class SetNetworkRBAC(common.NeutronCommandWithExtraArgs):
     _description = _("Set network RBAC policy properties")
 
     def get_parser(self, prog_name):
@@ -291,6 +294,8 @@ class SetNetworkRBAC(command.Command):
                 parsed_args.target_project_domain,
             ).id
             attrs['target_tenant'] = project_id
+        attrs.update(
+            self._parse_extra_properties(parsed_args.extra_properties))
         client.update_rbac_policy(obj, **attrs)
 
 
