@@ -23,11 +23,12 @@ from osc_lib.cli import parseractions
 from osc_lib.command import command
 from osc_lib import exceptions
 from osc_lib import utils
+from osc_lib.utils import tags as _tag
 
 from openstackclient.i18n import _
 from openstackclient.identity import common as identity_common
+from openstackclient.network import common
 from openstackclient.network import sdk_utils
-from openstackclient.network.v2 import _tag
 
 
 LOG = logging.getLogger(__name__)
@@ -49,7 +50,7 @@ class RouterInfoColumn(cliff_columns.FormattableColumn):
 class RoutesColumn(cliff_columns.FormattableColumn):
     def human_readable(self):
         # Map the route keys to match --route option.
-        for route in self._value:
+        for route in self._value or []:
             if 'nexthop' in route:
                 route['gateway'] = route.pop('nexthop')
         return utils.format_list_of_dicts(self._value)
@@ -61,7 +62,6 @@ _formatters = {
     'external_gateway_info': RouterInfoColumn,
     'availability_zones': format_columns.ListColumn,
     'availability_zone_hints': format_columns.ListColumn,
-    'location': format_columns.DictColumn,
     'routes': RoutesColumn,
     'tags': format_columns.ListColumn,
 }
@@ -168,9 +168,96 @@ class AddSubnetToRouter(command.Command):
             subnet_id=subnet.id)
 
 
+class AddExtraRoutesToRouter(command.ShowOne):
+    _description = _("Add extra static routes to a router's routing table.")
+
+    def get_parser(self, prog_name):
+        parser = super(AddExtraRoutesToRouter, self).get_parser(prog_name)
+        parser.add_argument(
+            'router',
+            metavar='<router>',
+            help=_("Router to which extra static routes "
+                   "will be added (name or ID).")
+        )
+        parser.add_argument(
+            '--route',
+            metavar='destination=<subnet>,gateway=<ip-address>',
+            action=parseractions.MultiKeyValueAction,
+            dest='routes',
+            default=[],
+            required_keys=['destination', 'gateway'],
+            help=_("Add extra static route to the router. "
+                   "destination: destination subnet (in CIDR notation), "
+                   "gateway: nexthop IP address. "
+                   "Repeat option to add multiple routes. "
+                   "Trying to add a route that's already present "
+                   "(exactly, including destination and nexthop) "
+                   "in the routing table is allowed and is considered "
+                   "a successful operation.")
+        )
+        return parser
+
+    def take_action(self, parsed_args):
+        if parsed_args.routes is not None:
+            for route in parsed_args.routes:
+                route['nexthop'] = route.pop('gateway')
+        client = self.app.client_manager.network
+        router_obj = client.add_extra_routes_to_router(
+            client.find_router(parsed_args.router, ignore_missing=False),
+            body={'router': {'routes': parsed_args.routes}})
+        display_columns, columns = _get_columns(router_obj)
+        data = utils.get_item_properties(
+            router_obj, columns, formatters=_formatters)
+        return (display_columns, data)
+
+
+class RemoveExtraRoutesFromRouter(command.ShowOne):
+    _description = _(
+        "Remove extra static routes from a router's routing table.")
+
+    def get_parser(self, prog_name):
+        parser = super(RemoveExtraRoutesFromRouter, self).get_parser(prog_name)
+        parser.add_argument(
+            'router',
+            metavar='<router>',
+            help=_("Router from which extra static routes "
+                   "will be removed (name or ID).")
+        )
+        parser.add_argument(
+            '--route',
+            metavar='destination=<subnet>,gateway=<ip-address>',
+            action=parseractions.MultiKeyValueAction,
+            dest='routes',
+            default=[],
+            required_keys=['destination', 'gateway'],
+            help=_("Remove extra static route from the router. "
+                   "destination: destination subnet (in CIDR notation), "
+                   "gateway: nexthop IP address. "
+                   "Repeat option to remove multiple routes. "
+                   "Trying to remove a route that's already missing "
+                   "(fully, including destination and nexthop) "
+                   "from the routing table is allowed and is considered "
+                   "a successful operation.")
+        )
+        return parser
+
+    def take_action(self, parsed_args):
+        if parsed_args.routes is not None:
+            for route in parsed_args.routes:
+                route['nexthop'] = route.pop('gateway')
+        client = self.app.client_manager.network
+        router_obj = client.remove_extra_routes_from_router(
+            client.find_router(parsed_args.router, ignore_missing=False),
+            body={'router': {'routes': parsed_args.routes}})
+        display_columns, columns = _get_columns(router_obj)
+        data = utils.get_item_properties(
+            router_obj, columns, formatters=_formatters)
+        return (display_columns, data)
+
+
 # TODO(yanxing'an): Use the SDK resource mapped attribute names once the
 # OSC minimum requirements include SDK 1.0.
-class CreateRouter(command.ShowOne):
+class CreateRouter(command.ShowOne, common.NeutronCommandWithExtraArgs):
     _description = _("Create a new router")
 
     def get_parser(self, prog_name):
@@ -246,6 +333,9 @@ class CreateRouter(command.ShowOne):
             attrs['ha'] = True
         if parsed_args.no_ha:
             attrs['ha'] = False
+        attrs.update(
+            self._parse_extra_properties(parsed_args.extra_properties))
+
         obj = client.create_router(**attrs)
         # tags cannot be set when created, so tags need to be set later.
         _tag.update_tags_for_set(client, obj, parsed_args)
@@ -490,7 +580,7 @@ class RemoveSubnetFromRouter(command.Command):
 
 # TODO(yanxing'an): Use the SDK resource mapped attribute names once the
 # OSC minimum requirements include SDK 1.0.
-class SetRouter(command.Command):
+class SetRouter(common.NeutronCommandWithExtraArgs):
     _description = _("Set router properties")
 
     def get_parser(self, prog_name):
@@ -540,17 +630,21 @@ class SetRouter(command.Command):
             dest='routes',
             default=None,
             required_keys=['destination', 'gateway'],
-            help=_("Routes associated with the router "
+            help=_("Add routes to the router "
                    "destination: destination subnet (in CIDR notation) "
                    "gateway: nexthop IP address "
-                   "(repeat option to set multiple routes)")
+                   "(repeat option to add multiple routes). "
+                   "This is deprecated in favor of 'router add/remove route' "
+                   "since it is prone to race conditions between concurrent "
+                   "clients when not used together with --no-route to "
+                   "overwrite the current value of 'routes'.")
         )
         parser.add_argument(
             '--no-route',
             action='store_true',
             help=_("Clear routes associated with the router. "
                    "Specify both --route and --no-route to overwrite "
-                   "current value of route.")
+                   "current value of routes.")
         )
         routes_ha = parser.add_mutually_exclusive_group()
         routes_ha.add_argument(
@@ -677,6 +771,10 @@ class SetRouter(command.Command):
 
         if 'no_qos_policy' in parsed_args and parsed_args.no_qos_policy:
             attrs['external_gateway_info']['qos_policy_id'] = None
+
+        attrs.update(
+            self._parse_extra_properties(parsed_args.extra_properties))
+
         if attrs:
             client.update_router(obj, **attrs)
         # tags is a subresource and it needs to be updated separately.
@@ -719,7 +817,7 @@ class ShowRouter(command.ShowOne):
         return (display_columns, data)
 
 
-class UnsetRouter(command.Command):
+class UnsetRouter(common.NeutronUnsetCommandWithExtraArgs):
     _description = _("Unset router properties")
 
     def get_parser(self, prog_name):
@@ -785,6 +883,10 @@ class UnsetRouter(command.Command):
 
         if parsed_args.external_gateway:
             attrs['external_gateway_info'] = {}
+
+        attrs.update(
+            self._parse_extra_properties(parsed_args.extra_properties))
+
         if attrs:
             client.update_router(obj, **attrs)
         # tags is a subresource and it needs to be updated separately.
