@@ -20,11 +20,12 @@ from osc_lib.cli import parseractions
 from osc_lib.command import command
 from osc_lib import exceptions
 from osc_lib import utils
+from osc_lib.utils import tags as _tag
 
 from openstackclient.i18n import _
 from openstackclient.identity import common as identity_common
+from openstackclient.network import common
 from openstackclient.network import sdk_utils
-from openstackclient.network.v2 import _tag
 
 
 LOG = logging.getLogger(__name__)
@@ -42,7 +43,6 @@ def _get_columns(item):
 
 
 _formatters = {
-    'location': format_columns.DictColumn,
     'prefixes': format_columns.ListColumn,
     'tags': format_columns.ListColumn,
 }
@@ -147,7 +147,7 @@ def _add_default_options(parser):
 
 # TODO(rtheis): Use the SDK resource mapped attribute names once the
 # OSC minimum requirements include SDK 1.0.
-class CreateSubnetPool(command.ShowOne):
+class CreateSubnetPool(command.ShowOne, common.NeutronCommandWithExtraArgs):
     _description = _("Create subnet pool")
 
     def get_parser(self, prog_name):
@@ -204,6 +204,8 @@ class CreateSubnetPool(command.ShowOne):
         # NeutronServer expects prefixes to be a List
         if "prefixes" not in attrs:
             attrs['prefixes'] = []
+        attrs.update(
+            self._parse_extra_properties(parsed_args.extra_properties))
         obj = client.create_subnet_pool(**attrs)
         # tags cannot be set when created, so tags need to be set later.
         _tag.update_tags_for_set(client, obj, parsed_args)
@@ -352,7 +354,7 @@ class ListSubnetPool(command.Lister):
 
 # TODO(rtheis): Use the SDK resource mapped attribute names once the
 # OSC minimum requirements include SDK 1.0.
-class SetSubnetPool(command.Command):
+class SetSubnetPool(common.NeutronCommandWithExtraArgs):
     _description = _("Set subnet pool properties")
 
     def get_parser(self, prog_name):
@@ -409,6 +411,9 @@ class SetSubnetPool(command.Command):
         if 'prefixes' in attrs:
             attrs['prefixes'].extend(obj.prefixes)
 
+        attrs.update(
+            self._parse_extra_properties(parsed_args.extra_properties))
+
         if attrs:
             client.update_subnet_pool(obj, **attrs)
         # tags is a subresource and it needs to be updated separately.
