@@ -13,9 +13,8 @@
 #
 
 import argparse
-
-import mock
-from mock import call
+from unittest import mock
+from unittest.mock import call
 
 from osc_lib.cli import format_columns
 from osc_lib import exceptions
@@ -42,8 +41,8 @@ class TestVolume(volume_fakes.TestVolume):
         self.users_mock = self.app.client_manager.identity.users
         self.users_mock.reset_mock()
 
-        self.images_mock = self.app.client_manager.image.images
-        self.images_mock.reset_mock()
+        self.find_image_mock = self.app.client_manager.image.find_image
+        self.find_image_mock.reset_mock()
 
         self.snapshots_mock = self.app.client_manager.volume.volume_snapshots
         self.snapshots_mock.reset_mock()
@@ -137,7 +136,7 @@ class TestVolumeCreate(TestVolume):
         )
 
         self.assertEqual(self.columns, columns)
-        self.assertItemEqual(self.datalist, data)
+        self.assertCountEqual(self.datalist, data)
 
     def test_volume_create_options(self):
         consistency_group = (
@@ -183,7 +182,7 @@ class TestVolumeCreate(TestVolume):
         )
 
         self.assertEqual(self.columns, columns)
-        self.assertItemEqual(self.datalist, data)
+        self.assertCountEqual(self.datalist, data)
 
     def test_volume_create_properties(self):
         arglist = [
@@ -219,11 +218,11 @@ class TestVolumeCreate(TestVolume):
         )
 
         self.assertEqual(self.columns, columns)
-        self.assertItemEqual(self.datalist, data)
+        self.assertCountEqual(self.datalist, data)
 
     def test_volume_create_image_id(self):
         image = image_fakes.FakeImage.create_one_image()
-        self.images_mock.get.return_value = image
+        self.find_image_mock.return_value = image
 
         arglist = [
             '--image', image.id,
@@ -257,11 +256,11 @@ class TestVolumeCreate(TestVolume):
         )
 
         self.assertEqual(self.columns, columns)
-        self.assertItemEqual(self.datalist, data)
+        self.assertCountEqual(self.datalist, data)
 
     def test_volume_create_image_name(self):
         image = image_fakes.FakeImage.create_one_image()
-        self.images_mock.get.return_value = image
+        self.find_image_mock.return_value = image
 
         arglist = [
             '--image', image.name,
@@ -295,7 +294,7 @@ class TestVolumeCreate(TestVolume):
         )
 
         self.assertEqual(self.columns, columns)
-        self.assertItemEqual(self.datalist, data)
+        self.assertCountEqual(self.datalist, data)
 
     def test_volume_create_with_snapshot(self):
         snapshot = volume_fakes.FakeSnapshot.create_one_snapshot()
@@ -332,7 +331,7 @@ class TestVolumeCreate(TestVolume):
         )
 
         self.assertEqual(self.columns, columns)
-        self.assertItemEqual(self.datalist, data)
+        self.assertCountEqual(self.datalist, data)
 
     def test_volume_create_with_bootable_and_readonly(self):
         arglist = [
@@ -370,7 +369,7 @@ class TestVolumeCreate(TestVolume):
         )
 
         self.assertEqual(self.columns, columns)
-        self.assertItemEqual(self.datalist, data)
+        self.assertCountEqual(self.datalist, data)
         self.volumes_mock.set_bootable.assert_called_with(
             self.new_volume.id, True)
         self.volumes_mock.update_readonly_flag.assert_called_with(
@@ -412,7 +411,7 @@ class TestVolumeCreate(TestVolume):
         )
 
         self.assertEqual(self.columns, columns)
-        self.assertItemEqual(self.datalist, data)
+        self.assertCountEqual(self.datalist, data)
         self.volumes_mock.set_bootable.assert_called_with(
             self.new_volume.id, False)
         self.volumes_mock.update_readonly_flag.assert_called_with(
@@ -464,7 +463,7 @@ class TestVolumeCreate(TestVolume):
 
         self.assertEqual(2, mock_error.call_count)
         self.assertEqual(self.columns, columns)
-        self.assertItemEqual(self.datalist, data)
+        self.assertCountEqual(self.datalist, data)
         self.volumes_mock.set_bootable.assert_called_with(
             self.new_volume.id, True)
         self.volumes_mock.update_readonly_flag.assert_called_with(
@@ -681,7 +680,7 @@ class TestVolumeList(TestVolume):
             self.mock_volume.size,
             volume.AttachmentsColumn(self.mock_volume.attachments),
         ), )
-        self.assertListItemEqual(datalist, tuple(data))
+        self.assertCountEqual(datalist, tuple(data))
 
     def test_volume_list_project(self):
         arglist = [
@@ -721,7 +720,7 @@ class TestVolumeList(TestVolume):
             self.mock_volume.size,
             volume.AttachmentsColumn(self.mock_volume.attachments),
         ), )
-        self.assertListItemEqual(datalist, tuple(data))
+        self.assertCountEqual(datalist, tuple(data))
 
     def test_volume_list_project_domain(self):
         arglist = [
@@ -763,7 +762,7 @@ class TestVolumeList(TestVolume):
             self.mock_volume.size,
             volume.AttachmentsColumn(self.mock_volume.attachments),
         ), )
-        self.assertListItemEqual(datalist, tuple(data))
+        self.assertCountEqual(datalist, tuple(data))
 
     def test_volume_list_user(self):
         arglist = [
@@ -802,7 +801,7 @@ class TestVolumeList(TestVolume):
             self.mock_volume.size,
             volume.AttachmentsColumn(self.mock_volume.attachments),
         ), )
-        self.assertListItemEqual(datalist, tuple(data))
+        self.assertCountEqual(datalist, tuple(data))
 
     def test_volume_list_user_domain(self):
         arglist = [
@@ -844,7 +843,7 @@ class TestVolumeList(TestVolume):
             self.mock_volume.size,
             volume.AttachmentsColumn(self.mock_volume.attachments),
         ), )
-        self.assertListItemEqual(datalist, tuple(data))
+        self.assertCountEqual(datalist, tuple(data))
 
     def test_volume_list_name(self):
         arglist = [
@@ -884,7 +883,7 @@ class TestVolumeList(TestVolume):
             self.mock_volume.size,
             volume.AttachmentsColumn(self.mock_volume.attachments),
         ), )
-        self.assertListItemEqual(datalist, tuple(data))
+        self.assertCountEqual(datalist, tuple(data))
 
     def test_volume_list_status(self):
         arglist = [
@@ -924,7 +923,7 @@ class TestVolumeList(TestVolume):
             self.mock_volume.size,
             volume.AttachmentsColumn(self.mock_volume.attachments),
         ), )
-        self.assertListItemEqual(datalist, tuple(data))
+        self.assertCountEqual(datalist, tuple(data))
 
     def test_volume_list_all_projects(self):
         arglist = [
@@ -964,7 +963,7 @@ class TestVolumeList(TestVolume):
             self.mock_volume.size,
             volume.AttachmentsColumn(self.mock_volume.attachments),
         ), )
-        self.assertListItemEqual(datalist, tuple(data))
+        self.assertCountEqual(datalist, tuple(data))
 
     def test_volume_list_long(self):
         arglist = [
@@ -1018,7 +1017,7 @@ class TestVolumeList(TestVolume):
             volume.AttachmentsColumn(self.mock_volume.attachments),
             format_columns.DictColumn(self.mock_volume.metadata),
         ), )
-        self.assertListItemEqual(datalist, tuple(data))
+        self.assertCountEqual(datalist, tuple(data))
 
     def test_volume_list_with_marker_and_limit(self):
         arglist = [
@@ -1057,7 +1056,7 @@ class TestVolumeList(TestVolume):
                 'name': None,
                 'all_tenants': False, }
         )
-        self.assertListItemEqual(datalist, tuple(data))
+        self.assertCountEqual(datalist, tuple(data))
 
     def test_volume_list_negative_limit(self):
         arglist = [
@@ -1174,7 +1173,7 @@ class TestVolumeMigrate(TestVolume):
 
 class TestVolumeSet(TestVolume):
 
-    volume_type = volume_fakes.FakeType.create_one_type()
+    volume_type = volume_fakes.FakeVolumeType.create_one_volume_type()
 
     def setUp(self):
         super(TestVolumeSet, self).setUp()
@@ -1451,7 +1450,7 @@ class TestVolumeShow(TestVolume):
             volume_fakes.FakeVolume.get_volume_columns(self._volume),
             columns)
 
-        self.assertItemEqual(
+        self.assertCountEqual(
             volume_fakes.FakeVolume.get_volume_data(self._volume),
             data)
 
